"use strict";

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var Format = require('./Format.js');

var _require = require('./utils.js'),
    toNumLevel = _require.toNumLevel,
    adjustLevel = _require.adjustLevel,
    LEVELS = _require.LEVELS,
    LOG = _require.LOG,
    INFO = _require.INFO,
    FATAL = _require.FATAL;

var Namespaces = require('./Namespaces.js');

var noop = function noop() {};

var time = {
  epoch: function epoch() {
    return Date.now();
  },
  unix: function unix() {
    return Date.now() / 1000 | 0;
  },
  iso: function iso() {
    return new Date().toISOString();
  }
};

function LogBase(name, opts) {
  Object.assign(this, {
    name: name,
    opts: opts,
    _enabled: {},
    formatter: new Format(opts)
  });
  this.serializers = Object.entries(this.opts.serializers || {}).reduce(function (o, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        val = _ref2[1];

    if (typeof val === 'function') {
      o = o || {};
      o[key] = val;
    }

    return o;
  }, null);
  this._timeF = time[opts.timestamp];
  this._time = this._timeF ? this._timeF : noop;
  this.enable();
}

LogBase.prototype = {
  enable: function enable(namespaces) {
    var _this = this;

    namespaces = namespaces || this.opts.namespaces;
    var namespace = new Namespaces(namespaces);
    this._enabled = {}; // reset

    var level = namespace.isEnabled(this.name, this.opts.level);
    var active = level ? LEVELS[adjustLevel(level, INFO)] : [];
    LEVELS.TRACE.forEach(function (level) {
      // loop over all levels
      var nlevel = _this.opts.levelNumbers ? toNumLevel(level) : level;
      var llevel = level.toLowerCase();

      if (active.includes(level)) {
        _this._enabled[level] = true;
        _this[llevel] = level === FATAL ? function (fmt) {
          for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          var str = _this._log(nlevel, fmt, args);

          _this.flush && _this.flush();
          return str;
        } : function (fmt) {
          for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }

          return _this._log(nlevel, fmt, args);
        };
      } else {
        _this[llevel] = noop;
      }
    });
    var nLOG = this.opts.levelNumbers ? toNumLevel(LOG) : LOG;

    this.log = function (fmt) {
      for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      return _this._log(nLOG, fmt, args);
    };
  },

  get enabled() {
    var _this2 = this;

    return this._enabled._cache || (this._enabled._cache = LEVELS.TRACE.reduce(function (o, level) {
      o[level] = o[level.toLowerCase()] = !!_this2._enabled[level];
      return o;
    }, {}));
  },

  diff: function diff() {
    var curr = Date.now();
    var prev = this._prev || curr;
    this._prev = curr;
    return curr - prev;
  },

  /**
   * @return {object} json object
   */
  _formatJson: function _formatJson(level, fmt) {
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    var isObject = _typeof(fmt) === 'object';
    var o = {
      level: level,
      time: this._time(),
      name: this.name,
      msg: isObject ? undefined : this.formatter.formatJson(fmt, args),
      pid: this.pid,
      hostname: this.hostname,
      diff: this.diff()
    };

    if (isObject) {
      (function () {
        var arg = fmt;

        do {
          var type = _typeof(arg);

          if (arg instanceof Error) {
            o.err = {
              name: arg.name,
              stack: arg.stack
            };
            o.msg = arg.message; // append other keys

            Object.keys(arg).forEach(function (key) {
              if (key === 'level') return;
              o.err[key] = arg[key];
            });
          } else if (arg && type === 'object') {
            if (Array.isArray(arg)) {
              o.arr = arg;
            } else {
              var _arg = arg,
                  name = _arg.name,
                  _level = _arg.level,
                  rest = _objectWithoutProperties(_arg, ["name", "level"]);

              Object.assign(o, rest);
            }
          } else {
            o.msg = arg;
            break;
          }

          arg = args.shift();
        } while (args.length);
      })();
    }

    return o;
  },
  _applySerializers: function _applySerializers(obj) {
    var o = {};

    for (var key in obj) {
      var value = obj[key];

      if (Object.prototype.hasOwnProperty.call(obj, key) && value !== undefined) {
        if (this.serializers && this.serializers[key]) {
          o[key] = this.serializers[key](value);
        } else {
          o[key] = value;
        }
      }
    }

    return o;
  },

  /* c8 ignore next */
  _serverinfo: function _serverinfo() {},

  /* c8 ignore next 3 */
  _log: function _log()
  /* level, args */
  {
    throw new Error('needs implementation');
  }
};
module.exports = LogBase;